//! Raw bindings to JavaScript objects used
//! by a [`TransformStream`](https://developer.mozilla.org/en-US/docs/Web/API/TransformStream).
//! These are re-exported from [web-sys](https://docs.rs/web-sys/0.3.70/web_sys/struct.TransformStream.html).
pub use web_sys::TransformStream;
