//! Provides a host of special statistical functions (e.g. the beta function or
//! the error function)

pub mod beta;
pub mod erf;
pub mod evaluate;
pub mod exponential;
pub mod factorial;
pub mod gamma;
pub mod harmonic;
pub mod logistic;
